@@ -6,10 +6,9 @@
 #[macro_use]
 extern crate alloc;
 
-#[cfg(feature = "experimental")]
 pub mod channel;
+pub mod errors;
 pub mod eth;
-#[cfg(feature = "experimental")]
 pub mod event_bus;
 #[cfg(all(feature = "experimental", feature = "alloc"))]
 pub mod http;
@@ -17,18 +16,16 @@ pub mod http;
 pub mod httpd;
 pub mod io;
 pub mod ipv4;
-#[cfg(all(feature = "experimental", feature = "alloc"))]
+#[cfg(feature = "alloc")]
 pub mod mqtt;
 pub mod mutex;
 #[cfg(all(feature = "experimental", feature = "alloc"))]
 pub mod ota;
 pub mod ping;
-pub mod service;
 #[cfg(feature = "alloc")]
 pub mod storage;
 pub mod sys_time;
-#[cfg(feature = "experimental")]
 pub mod timer;
+pub mod unblocker;
 pub mod utils;
-#[cfg(feature = "alloc")]
 pub mod wifi;
