@@ -1,6 +1,3 @@
-#[cfg(feature = "alloc")]
-pub mod anyerror;
-
 // Uncomment once domain 0.6.2 which has no_std support is released
 //#[cfg(feature = "experimental")]
 //pub mod captive;
@@ -8,9 +5,6 @@ pub mod anyerror;
 #[cfg(feature = "experimental")]
 pub mod nonblocking;
 
-#[cfg(feature = "experimental")]
-pub mod pinned_timer;
-
 #[cfg(all(feature = "experimental", feature = "alloc", feature = "use_serde"))]
 pub mod ghota;
 
